@@ -1,7 +1,7 @@
 package Data::Clean;
 
 our $DATE = '2016-05-21'; # DATE
-our $VERSION = '0.34'; # VERSION
+our $VERSION = '0.35'; # VERSION
 
 use 5.010001;
 use strict;
@@ -45,7 +45,7 @@ sub command_deref_scalar {
 
 sub command_stringify {
     my ($self, $args) = @_;
-    return '{{var}} = "{{var}}"';
+    return '{{var}} = "{{var}}"; $ref = ""';
 }
 
 sub command_replace_with_ref {
@@ -153,13 +153,6 @@ sub _generate_cleanser_code {
         $add_new_if->("\$ref eq '$ref'", $act0);
     };
 
-    # recurse into hash- or array-based object if !recurse_obj option is true
-    if ($opts->{'!recurse_obj'}) {
-        # XXX optimize, avoid calling reftype() twice
-        $add_if->('$reftype eq "ARRAY"', '$process_array->({{var}})');
-        $add_if->('$reftype eq "HASH"' , '$process_hash->({{var}})');
-    }
-
     # catch object of specified classes (e.g. DateTime, etc)
     for my $on (grep {/\A\w*(::\w+)*\z/} sort keys %$opts) {
         my $o = $opts->{$on};
@@ -193,8 +186,14 @@ sub _generate_cleanser_code {
     }
 
     # recurse array and hash
-    $add_if_ref->("ARRAY", '$process_array->({{var}})');
-    $add_if_ref->("HASH" , '$process_hash->({{var}})');
+    if ($opts->{'!recurse_obj'}) {
+        $add_stmt->('stmt', 'my $reftype=Scalar::Util::reftype({{var}})//""');
+        $add_new_if->('$reftype eq "ARRAY"', '$process_array->({{var}})');
+        $add_if->('$reftype eq "HASH"' , '$process_hash->({{var}})');
+    } else {
+        $add_new_if_ref->("ARRAY", '$process_array->({{var}})');
+        $add_if_ref->("HASH" , '$process_hash->({{var}})');
+    }
 
     # lastly, catch any reference left
     for my $p ([-ref => '$ref']) {
@@ -215,21 +214,18 @@ sub _generate_cleanser_code {
     push @code, 'state $process_hash;'."\n";
     push @code, (
         'if (!$process_array) { $process_array = sub { my $a = shift; for my $e (@$a) { ',
-        'my $ref=ref($e);',
-        ($opts->{'!recurse_obj'} ? ' my $reftype=Scalar::Util::reftype($e)//"";':''), "\n",
+        'my $ref=ref($e);'."\n",
         join("", @stmts_ary).'} } }'."\n"
     );
     push @code, (
         'if (!$process_hash) { $process_hash = sub { my $h = shift; for my $k (keys %$h) { ',
-        'my $ref=ref($h->{$k});',
-        ($opts->{'!recurse_obj'} ? ' my $reftype=Scalar::Util::reftype($h->{$k})//"";':''), "\n",
+        'my $ref=ref($h->{$k});'."\n",
         join("", @stmts_hash).'} } }'."\n"
     );
     push @code, '%refs = (); $ctr_circ=0;'."\n" if $circ;
     push @code, (
         'for ($data) { ',
-        'my $ref=ref($_);',
-        ($opts->{'!recurse_obj'} ? ' my $reftype=Scalar::Util::reftype($_)//"";':''), "\n",
+        'my $ref=ref($_);'."\n",
         join("", @stmts_main).'}'."\n"
     );
     push @code, '$data'."\n";
@@ -276,7 +272,7 @@ Data::Clean - Clean data structure
 
 =head1 VERSION
 
-This document describes version 0.34 of Data::Clean (from Perl distribution Data-Clean), released on 2016-05-21.
+This document describes version 0.35 of Data::Clean (from Perl distribution Data-Clean), released on 2016-05-21.
 
 =head1 SYNOPSIS
 
